#include <stdio.h>
#include <string.h>

// ----------------------------------------------------------------------------
// IHaveDependencyWith
// ----------------------------------------------------------------------------
template <class T>
class IHaveDependencyWith
{
public:
  /// class T must have static void dependency()
  IHaveDependencyWith()          { T::dependency(); }
  virtual ~IHaveDependencyWith() {}
};

// ----------------------------------------------------------------------------
// Log
// ----------------------------------------------------------------------------
class Log
{
  FILE* fp;
private:
  Log()
  { 
    fp = fopen("test.log", "wb");
  }
  virtual ~Log()
  {
    fclose(fp);
    fp = NULL;
  }
public:
  void write(const char* msg)
  {
    fwrite(msg, 1, strlen(msg), fp);
  };
public:
  static Log& instance()
  {
    static Log g_log;
    return g_log;
  }
public:
  static void dependency()
  {
    instance();
  }
};

// ----------------------------------------------------------------------------
// App
// ----------------------------------------------------------------------------
class App : private IHaveDependencyWith<Log>
{
private:
  App()
  {
    //Log::instance().write("App::App\n");
  }
  virtual ~App()
  {
    Log::instance().write("App::~App\n");
  }
public:
  void start() {
    Log::instance().write("App::start\n");
  }
  static App& instance()
  {
    static App g_app;
    return g_app;
  }
};

// ----------------------------------------------------------------------------
// main
// ----------------------------------------------------------------------------
int main()
{
  App::instance().start();
  return 0;
}
